@@ -21,6 +21,7 @@
 #include "oxygencomboboxdata.h"
 #include "../oxygengtkutils.h"
 #include "../config.h"
+#include "../oxygenstyle.h"
 
 #include <gtk/gtk.h>
 #include <iostream>
@@ -40,6 +41,12 @@ namespace Oxygen
         _stateChangeId.connect( G_OBJECT(widget), "state-changed", G_CALLBACK( stateChangeEvent ), this );
         initializeCellView( widget );
 
+        /*
+        need to force the wrap-width property to 0,
+        otherwise the "appears-as-list" flag is not respected, which additionaly breaks the widget rendering.
+        */
+        gtk_combo_box_set_wrap_width( GTK_COMBO_BOX( widget ), 0 );
+
     }
 
     //________________________________________________________________________________
@@ -143,7 +150,10 @@ namespace Oxygen
             gtk_cell_renderer_get_size( r, _cell._widget, 0L, 0L, 0L, &xsize, &ysize );
 
             gtk_cell_renderer_set_padding( r, std::max( 6, xpad ), ypadmax );
-            gtk_cell_renderer_set_fixed_size( r, xsize+6, ysize + ypadmax );
+
+            if( GTK_IS_CELL_RENDERER_PIXBUF( r ) )
+            { gtk_cell_renderer_set_fixed_size( r, xsize+6, ysize + ypadmax ); }
+
         }
 
         if( renderers ) g_list_free( renderers );
