@@ -24,6 +24,7 @@
 */
 
 #include "oxygenapplicationname.h"
+#include "oxygengtkutils.h"
 #include "config.h"
 
 #include <iostream>
@@ -36,16 +37,34 @@ namespace Oxygen
     {
 
         #if OXYGEN_DEBUG
-        std::cout << "ApplicationName::parse - " << appName << std::endl;
+        std::cerr << "ApplicationName::parse - " << appName << std::endl;
         #endif
 
         if( appName.find("firefox") == 0 ) _name = Firefox;
+        else if( appName.find("xulrunner") == 0 ) _name = Xul;
         else if( appName.find("thunderbird") == 0 ) _name = Thunderbird;
         else if( appName.find("seamonkey" ) == 0 ) _name = Seamonkey;
         else if( appName == "soffice" ) _name = OpenOffice;
         else if( appName == "gimp" ) _name = Gimp;
-        else if( appName == "chromium-browser" || appName == "google-chrome" ) _name = GoogleChrome;
+        else if( appName == "chromium" || appName == "chromium-browser" || appName == "google-chrome" ) _name = GoogleChrome;
         else _name = Unknown;
     }
 
+    //__________________________________________________________________________
+    bool ApplicationName::isMozilla( GtkWidget* widget ) const
+    {
+        if( !isMozilla() ) return false;
+
+        GtkWidget* parent( gtk_widget_get_toplevel( widget ) );
+
+        #if OXYGEN_DEBUG
+        if( parent ) std::cerr << "Oxygen::ApplicationName::isMozilla - parent: " << G_OBJECT_TYPE_NAME( widget ) << std::endl;
+        #endif
+
+        // check parent
+        if( parent && GTK_IS_DIALOG( parent ) ) return false;
+        else return true;
+
+    }
+
 }
