@@ -25,6 +25,7 @@
 * MA 02110-1301, USA.
 */
 
+#include <gtk/gtk.h>
 #include <string>
 
 namespace Oxygen
@@ -37,6 +38,7 @@ namespace Oxygen
         Firefox,
         Seamonkey,
         Thunderbird,
+        Xul,
         Gimp,
         OpenOffice,
         GoogleChrome
@@ -78,12 +80,17 @@ namespace Oxygen
         bool isFirefox( void ) const { return _name == Firefox; }
         bool isThunderbird( void ) const { return _name == Thunderbird; }
         bool isSeamonkey( void ) const { return _name == Seamonkey; }
-        bool isMozilla( void ) const { return isFirefox() || isThunderbird() || isSeamonkey(); }
+        bool isXul( void ) const { return _name == Xul; }
+        bool isMozilla( void ) const { return isFirefox() || isThunderbird() || isSeamonkey() || isXul(); }
         bool isGimp( void ) const { return _name == Gimp; }
         bool isOpenOffice( void ) const { return _name == OpenOffice; }
         bool isGoogleChrome( void ) const { return _name == GoogleChrome; }
         bool isUnknown( void ) const { return _name == Unknown; }
 
+        //! special case for mozilla that also check the type of the top level widget
+        /*! this allows to prevent false positive for open and print dialogs */
+        bool isMozilla( GtkWidget* ) const;
+
         //@}
 
         private:
