@@ -23,6 +23,8 @@
 */
 
 #include "oxygengtkutils.h"
+#include "oxygengtktypenames.h"
+#include "config.h"
 
 #include <cmath>
 #include <cstring>
@@ -57,14 +59,21 @@ namespace Oxygen
 
     }
 
+    //____________________________________________________________
+    bool Gtk::gtk_widget_is_panel_applet( GtkWidget* widget )
+    {
+        std::string name( G_OBJECT_TYPE_NAME( widget ) );
+        return name.find( "PanelApplet" ) == 0 || name.find( "PanelWidget" ) == 0;
+    }
+
     //____________________________________________________________
     void Gtk::gtk_widget_print_tree( GtkWidget* widget )
     {
 
         if( !widget ) return;
-        std::cout << "Oxygen::Gtk::gtk_widget_print_tree - widget: " << widget << " (" << G_OBJECT_TYPE_NAME( widget ) << ")" << std::endl;
+        std::cerr << "Oxygen::Gtk::gtk_widget_print_tree - widget: " << widget << " (" << G_OBJECT_TYPE_NAME( widget ) << ")" << std::endl;
         while( ( widget = gtk_widget_get_parent( widget ) ) )
-        { std::cout << "    parent: " << widget << " (" << G_OBJECT_TYPE_NAME( widget ) << ")" << std::endl; }
+        { std::cerr << "    parent: " << widget << " (" << G_OBJECT_TYPE_NAME( widget ) << ")" << std::endl; }
 
     }
 
@@ -75,7 +84,7 @@ namespace Oxygen
         if( !widget ) return false;
 
         GdkScreen *screen( gtk_widget_get_screen( widget ) );
-        if( gdk_screen_is_composited( screen ) )
+        if( screen && gdk_screen_is_composited( screen ) )
         {
 
             GdkVisual *visual( gtk_widget_get_visual (widget) );
@@ -92,15 +101,29 @@ namespace Oxygen
     //________________________________________________________
     bool Gtk::gdk_window_is_base( GdkWindow* window )
     {
+
+        if( !GDK_IS_WINDOW( window ) ) return false;
+
         GdkWindowTypeHint hint = gdk_window_get_type_hint( window );
-        return( hint == GDK_WINDOW_TYPE_HINT_NORMAL || hint == GDK_WINDOW_TYPE_HINT_DIALOG );
+
+        #if OXYGEN_DEBUG
+        std::cerr << "Gtk::gdk_window_is_base - " << TypeNames::windowTypeHint( hint ) << std::endl;
+        #endif
+
+        return(
+            hint == GDK_WINDOW_TYPE_HINT_NORMAL ||
+            hint == GDK_WINDOW_TYPE_HINT_DIALOG ||
+            hint == GDK_WINDOW_TYPE_HINT_UTILITY );
     }
 
     //________________________________________________________
     bool Gtk::gdk_window_nobackground( GdkWindow* window )
     {
+        if( !GDK_IS_WINDOW( window ) ) return false;
+
         GdkWindowTypeHint hint = gdk_window_get_type_hint( window );
         return( hint == GDK_WINDOW_TYPE_HINT_COMBO || hint == GDK_WINDOW_TYPE_HINT_TOOLTIP );
+
     }
 
     //________________________________________________________
@@ -249,7 +272,7 @@ namespace Oxygen
     }
 
     //________________________________________________________
-    bool Gtk::gtk_is_parent( GtkWidget* widget, GtkWidget* potentialParent )
+    bool Gtk::gtk_widget_is_parent( GtkWidget* widget, GtkWidget* potentialParent )
     {
 
         GtkWidget *parent( widget );
@@ -291,31 +314,72 @@ namespace Oxygen
     //________________________________________________________
     GtkWidget* Gtk::gtk_button_find_image(GtkWidget* button)
     {
-        if(!GTK_IS_CONTAINER(button))
-            return 0L;
-        for(GList* children=gtk_container_get_children(GTK_CONTAINER(button)); children; children=children->next)
+
+        // check widget type
+        if(!GTK_IS_CONTAINER(button)) return 0L;
+
+        GtkWidget* result( 0L );
+        GList* children( gtk_container_get_children( GTK_CONTAINER( button ) ) );
+        for( GList *child = g_list_first( children ); child; child = g_list_next( child ) )
         {
-            if(GTK_IS_IMAGE(children->data))
-                return GTK_WIDGET(children->data);
-            else if(GTK_IS_CONTAINER(children->data))
-                return gtk_button_find_image(GTK_WIDGET(children->data));
+            if( GTK_IS_IMAGE( child->data ) )
+            {
+                result = GTK_WIDGET( child->data );
+                break;
+
+            } else if( GTK_IS_CONTAINER( child->data ) ) {
+
+                result = gtk_button_find_image( GTK_WIDGET(child->data ) );
+                break;
+
+            }
+
         }
-        return 0L;
+
+        if( children ) g_list_free( children );
+        return result;
+
     }
 
     //________________________________________________________
     GtkWidget* Gtk::gtk_button_find_label(GtkWidget* button)
     {
-        if(!GTK_IS_CONTAINER(button))
-            return 0L;
-        for(GList* children=gtk_container_get_children(GTK_CONTAINER(button)); children; children=children->next)
+
+        // check widget type
+        if( !GTK_IS_CONTAINER(button) ) return 0L;
+
+        GtkWidget* result( 0L );
+        GList* children( gtk_container_get_children( GTK_CONTAINER( button ) ) );
+        for( GList *child = g_list_first( children ); child; child = g_list_next( child ) )
         {
-            if(GTK_IS_LABEL(children->data))
-                return GTK_WIDGET(children->data);
-            else if(GTK_IS_CONTAINER(children->data))
-                return gtk_button_find_image(GTK_WIDGET(children->data));
+            if( GTK_IS_LABEL( child->data) )
+            {
+                result = GTK_WIDGET( child->data );
+                break;
+
+            } else if( GTK_IS_CONTAINER( child->data ) ) {
+
+                result = gtk_button_find_image(GTK_WIDGET(child->data));
+                break;
+
+            }
+
         }
-        return 0L;
+
+        if( children ) g_list_free( children );
+        return result;
+
+    }
+
+    //________________________________________________________
+    bool Gtk::gtk_combobox_has_frame( GtkWidget* widget )
+    {
+
+        GValue val = { 0, };
+        g_value_init(&val, G_TYPE_BOOLEAN);
+        g_object_get_property( G_OBJECT( widget ), "has-frame", &val );
+        return (bool) g_value_get_boolean( &val );
+
     }
 
     //________________________________________________________
@@ -446,6 +510,119 @@ namespace Oxygen
 
     }
 
+    //____________________________________________________________
+    bool Gtk::gtk_notebook_is_tab_label(GtkNotebook* notebook, GtkWidget* widget )
+    {
+
+        for( int i = 0; i <  gtk_notebook_get_n_pages( notebook ); ++i )
+        {
+            // retrieve page and tab label
+            GtkWidget* page( gtk_notebook_get_nth_page( notebook, i ) );
+            GtkWidget* tabLabel( gtk_notebook_get_tab_label( notebook, page ) );
+            if( widget == tabLabel ) return true;
+        }
+
+        return false;
+
+    }
+
+
+    //____________________________________________________________
+    void Gtk::gtk_notebook_get_tabbar_rect( GtkNotebook* notebook, GdkRectangle* rect )
+    {
+        // check notebook and rect
+        if( !( notebook && rect ) ) return;
+
+        // check tab visibility
+        GList* children( gtk_container_get_children( GTK_CONTAINER( notebook ) ) );
+        if( !( gtk_notebook_get_show_tabs( notebook ) && children ) )
+        {
+            if( children ) g_list_free( children );
+            *rect = gdk_rectangle();
+            return;
+        }
+
+        // free children
+        if( children ) g_list_free( children );
+
+        // get full rect
+        gtk_widget_get_allocation( GTK_WIDGET( notebook ), rect );
+
+        // adjust to account for borderwidth
+        guint borderWidth( gtk_container_get_border_width( GTK_CONTAINER( notebook ) ) );
+        rect->x += borderWidth;
+        rect->y += borderWidth;
+        rect->height -= 2*borderWidth;
+        rect->width -= 2*borderWidth;
+
+        // get current page
+        int pageIndex( gtk_notebook_get_current_page( notebook ) );
+        if( !( pageIndex >= 0 && pageIndex < gtk_notebook_get_n_pages( notebook ) ) )
+        {
+            *rect = gdk_rectangle();
+            return;
+        }
+
+        GtkWidget* page( gtk_notebook_get_nth_page( notebook, pageIndex ) );
+        if( !page )
+        {
+            *rect = gdk_rectangle();
+            return;
+        }
+
+        // removes page allocated size from rect, based on tabwidget orientation
+        const GtkAllocation& pageAllocation( page->allocation );
+        switch( gtk_notebook_get_tab_pos( notebook ) )
+        {
+            case GTK_POS_BOTTOM:
+            rect->y += pageAllocation.height;
+            rect->height -= pageAllocation.height;
+            break;
+
+            case GTK_POS_TOP:
+            rect->height -= pageAllocation.height;
+            break;
+
+            case GTK_POS_RIGHT:
+            rect->x += pageAllocation.width;
+            rect->width -= pageAllocation.width;
+            break;
+
+            case GTK_POS_LEFT:
+            rect->width -= pageAllocation.width;
+            break;
+        }
+
+        return;
+
+    }
+
+    //____________________________________________________________
+    bool Gtk::gtk_notebook_has_visible_arrows( GtkNotebook* notebook )
+    {
+
+        if( !gtk_notebook_get_show_tabs( notebook ) ) return false;
+
+        // loop over pages
+        for( int i = 0; i <  gtk_notebook_get_n_pages( notebook ); ++i )
+        {
+
+            // retrieve page and tab label
+            GtkWidget* page( gtk_notebook_get_nth_page( notebook, i ) );
+            GtkWidget* label( gtk_notebook_get_tab_label( notebook, page ) );
+
+            #if GTK_CHECK_VERSION(2, 20, 0)
+            if( label && !gtk_widget_get_mapped( label ) ) return true;
+            #else
+            if( label && !GTK_WIDGET_MAPPED( label ) ) return true;
+            #endif
+        }
+
+        return false;
+
+    }
+
+
     //____________________________________________________________
     int Gtk::gtk_notebook_get_current_tab(GtkNotebook* notebook)
     {
@@ -454,7 +631,7 @@ namespace Oxygen
     }
 
     //____________________________________________________________
-    gboolean Gtk::gtk_notebook_update_close_buttons(GtkNotebook* notebook)
+    bool Gtk::gtk_notebook_update_close_buttons(GtkNotebook* notebook)
     {
         // cast to notebook and check against number of pages
         if( GTK_IS_NOTEBOOK( notebook ) )
@@ -480,40 +657,38 @@ namespace Oxygen
     //________________________________________________________
     bool Gtk::gtk_notebook_is_close_button(GtkWidget* widget)
     {
-        if(GtkNotebook* nb=GTK_NOTEBOOK(gtk_parent_notebook(widget)))
+        if( GtkNotebook* nb=GTK_NOTEBOOK(gtk_parent_notebook(widget) ) )
         {
             // check if the button resides on tab label, not anywhere on the tab
             bool tabLabelIsParent=false;
-            for(int i=0;i<gtk_notebook_get_n_pages(nb);++i)
+            for( int i=0; i < gtk_notebook_get_n_pages(nb); ++i )
             {
-                GtkWidget* tabLabel=gtk_notebook_get_tab_label(nb,gtk_notebook_get_nth_page(nb,i));
-                if(gtk_is_parent(widget,GTK_WIDGET(tabLabel)))
-                {
-                    tabLabelIsParent=true;
-                }
+                GtkWidget* tabLabel( gtk_notebook_get_tab_label(nb,gtk_notebook_get_nth_page( nb, i ) ) );
+                if( gtk_widget_is_parent( widget, GTK_WIDGET(tabLabel) ) )
+                { tabLabelIsParent=true; }
             }
 
-            if(!tabLabelIsParent) return false;
+            if( !tabLabelIsParent ) return false;
 
             // make sure button has no text and some image (for now, just hope it's a close icon)
-            if(!gtk_button_find_image(widget) || gtk_button_get_label(GTK_BUTTON(widget)))
+            if( gtk_button_find_image(widget) && !gtk_button_get_label( GTK_BUTTON(widget) ) )
+            { return true; }
+
+            // check for pidgin 'x' close button
+            if( GtkWidget* label = gtk_button_find_label(widget) )
             {
-                // check for pidgin 'x' close button
-                GtkWidget* label;
-                if(!(label=gtk_button_find_label(widget))) return false;
-                else {
 
-                    const gchar* labelText=gtk_label_get_text(GTK_LABEL(label));
-                    if(!strcmp(labelText,"×")) // It's not letter 'x' - it's a special symbol
-                    {
-                        gtk_widget_hide(label);
-                        return true;
-                    } else return false;
-                }
+                const gchar* labelText=gtk_label_get_text( GTK_LABEL(label) );
+                if(!strcmp(labelText,"×")) // It's not letter 'x' - it's a special symbol
+                {
+                    gtk_widget_hide( label );
+                    return true;
+                } else return false;
 
-            } else return true;
+            } else return false;
 
         } else return false;
+
     }
 
     //________________________________________________________
