@@ -55,6 +55,9 @@ namespace Oxygen
         //! set all buttons in the container to state NORMAL
         void gtk_container_adjust_buttons_state( GtkContainer*, gpointer=0L );
 
+        //! returns true if is an Gnome applet
+        bool gtk_widget_is_panel_applet( GtkWidget* );
+
         //! print some widget information
         void gtk_widget_print_tree( GtkWidget* );
 
@@ -114,7 +117,7 @@ namespace Oxygen
         GtkWidget* gtk_parent_notebook( GtkWidget* );
 
         //! returns true if potentialParent is (maybe indirect) parent of widget
-        bool gtk_is_parent( GtkWidget*, GtkWidget* potentialParent );
+        bool gtk_widget_is_parent( GtkWidget*, GtkWidget* potentialParent );
 
         //! returns true if one of the parent widgets has a sunken shadow
         bool gtk_parent_is_shadow_in( GtkWidget* );
@@ -145,6 +148,9 @@ namespace Oxygen
         //!@name combobox utilities
         //@{
 
+        //! returns true if combobox has frame
+        bool gtk_combobox_has_frame( GtkWidget* );
+
         //! true if widget is the treeview of a combobox
         bool gtk_combobox_is_tree_view( GtkWidget* );
 
@@ -175,11 +181,21 @@ namespace Oxygen
         //! returns index of first visible tab
         int gtk_notebook_find_first_tab( GtkWidget* );
 
+        //! returns true if widget is one of the notebook's tab labels
+        bool gtk_notebook_is_tab_label( GtkNotebook*, GtkWidget* );
+
+        //! returns tabbar rect in notebook
+        void gtk_notebook_get_tabbar_rect( GtkNotebook*, GdkRectangle* );
+
+        //! returns true if notebook has visible scrollbar arrows
+        /*! arrows are dimmed visible if at least one of the child tab_labels is unmapped */
+        bool gtk_notebook_has_visible_arrows( GtkNotebook* );
+
         //! index of current page
         int gtk_notebook_get_current_tab( GtkNotebook* );
 
         // make all the buttons on the tabs normal
-        gboolean gtk_notebook_update_close_buttons( GtkNotebook*);
+        bool gtk_notebook_update_close_buttons( GtkNotebook*);
 
         //! returns true if widget is a notebook close button
         bool gtk_notebook_is_close_button( GtkWidget* );
