@@ -3,11 +3,6 @@
 * Copyright (c) 2010 Hugo Pereira Da Costa <hugo@oxygen-icons.org>
 * Copyright (c) 2010 Ruslan Kabatsayev <b7.10110111@gmail.com>
 *
-* inspired notably from kdelibs/kdeui/color/kcolorutils.h
-* Copyright (C) 2007 Matthew Woehlke <mw_triad@users.sourceforge.net>
-* Copyright (C) 2007 Thomas Zander <zander@kde.org>
-* Copyright (C) 2007 Zack Rusin <zack@kde.org>
-*
 * This  library is free  software; you can  redistribute it and/or
 * modify it  under  the terms  of the  GNU Lesser  General  Public
 * License  as published  by the Free  Software  Foundation; either
@@ -28,14 +23,15 @@
 #include "oxygencoloreffect.h"
 #include "oxygencolorutils.h"
 #include "oxygenfontinfo.h"
-#include "oxygengtkicons.h"
 #include "config.h"
 
 #include <gtk/gtk.h>
 
+#include <algorithm>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#include <sys/stat.h>
 #include <fstream>
 #include <iostream>
 #include <sstream>
@@ -70,13 +66,19 @@ namespace Oxygen
         _startDragDist( 4 ),
         _startDragTime( 500 ),
         _buttonSize( ButtonDefault ),
-        _frameBorder( BorderDefault )
+        _frameBorder( BorderDefault ),
+        _initialized( false ),
+        _kdeColorsInitialized( false ),
+        _gtkColorsInitialized( false )
     {}
 
     //_________________________________________________________
     void QtSettings::initialize( void )
     {
 
+        if( _initialized ) return;
+        _initialized = true;
+
         // clear RC
         _rc.clear();
 
@@ -91,7 +93,7 @@ namespace Oxygen
         for( PathList::const_reverse_iterator iter = _kdeConfigPathList.rbegin(); iter != _kdeConfigPathList.rend(); ++iter )
         {
             #if OXYGEN_DEBUG
-            std::cout << "QtSettings::initialize - reading config from: " << *iter << std::endl;
+            std::cerr << "QtSettings::initialize - reading config from: " << *iter << std::endl;
             #endif
 
             _kdeGlobals.merge( readOptions( sanitizePath( *iter + "/kdeglobals" ) ) );
@@ -100,84 +102,87 @@ namespace Oxygen
         }
 
         #if OXYGEN_DEBUG
-        std::cout << "QtSettings::initialize - Kdeglobals: " << std::endl;
-        std::cout << _kdeGlobals << std::endl;
+        std::cerr << "QtSettings::initialize - Kdeglobals: " << std::endl;
+        std::cerr << _kdeGlobals << std::endl;
 
-        std::cout << "QtSettings::initialize - Oxygenrc: " << std::endl;
-        std::cout << _oxygen << std::endl;
+        std::cerr << "QtSettings::initialize - Oxygenrc: " << std::endl;
+        std::cerr << _oxygen << std::endl;
         #endif
 
-        // reload palette
-        loadKdePalette();
-
-        // reload icons
-        #if OXYGEN_ICON_HACK
-        _kdeIconPathList = kdeIconPathList();
-        loadKdeIcons();
-        #endif
-
-        // reload fonts
-        loadKdeFonts();
-
         // kde globals options
         loadKdeGlobalsOptions();
 
         // oxygen options
         loadOxygenOptions();
 
+        // reload fonts
+        loadKdeFonts();
+
+        // color palette
+        loadKdePalette();
+
         // gtk colors
         generateGtkColors();
 
-        #if OXYGEN_DEBUG
-        std::cout << "Oxygen::QtSettings::initialize - Gtkrc: " << std::endl;
-        std::cout << _rc << std::endl;
+        // reload icons
+        #if OXYGEN_ICON_HACK
+        _kdeIconPathList = kdeIconPathList();
+        loadKdeIcons();
         #endif
 
-        // pass all resources to gtk
-        gtk_rc_parse_string( _rc.toString().c_str() );
-
         #if OXYGEN_DEBUG
-        std::cout << "Oxygen::QtSettings::initialize - done. " << std::endl;
+        std::cerr << "Oxygen::QtSettings::initialize - Gtkrc: " << std::endl;
+        std::cerr << _rc << std::endl;
         #endif
 
-    }
+        // pass all resources to gtk and clear
+        _rc.commit();
 
-    //_________________________________________________________
-    void QtSettings::PathList::split( const std::string& path, const std::string& separator )
-    {
+        #if OXYGEN_DEBUG
+        std::cerr << "Oxygen::QtSettings::initialize - done. " << std::endl;
+        #endif
 
-        clear();
-        std::string local( path );
-        if( local.empty() ) return;
-        if( local[local.size()-1] == '\n' ) local = local.substr( 0, local.size()-1 );
-
-        size_t position( std::string::npos );
-        while( ( position = local.find( separator ) ) != std::string::npos )
-        {
-            push_back( local.substr(0, position ) );
-            local = local.substr( position + separator.length() );
-        }
-
-        if( !local.empty() ) push_back( local );
         return;
 
     }
 
     //_________________________________________________________
-    std::string QtSettings::PathList::join( const std::string& separator ) const
+    void QtSettings::initializeColors( void )
     {
-        std::ostringstream out;
-        for( const_iterator iter = begin(); iter != end(); ++iter )
-        {
-            if( iter != begin() ) out << separator;
-            out << *iter;
-        }
 
-        return out.str();
+        // reload palette, if needed
+        loadKdePalette();
+
+        if( _gtkColorsInitialized ) return;
+        _gtkColorsInitialized = true;
+
+        #if OXYGEN_DEBUG
+        std::cerr << "Oxygen::QtSettings::initializeColors." << std::endl;
+        #endif
+
+        // clear rc and generate gtk colors
+        _rc.clear();
+
+        // for some reason one also needs to rewrite the icon part
+        loadKdeIcons();
+
+        // generate gtk colors
+        generateGtkColors();
+
+        #if OXYGEN_DEBUG
+        std::cerr << "Oxygen::QtSettings::initializeColors - Gtkrc: " << std::endl;
+        std::cerr << _rc << std::endl;
+        #endif
+
+        // pass all resources to gtk and clear
+        _rc.commit();
+
+        return;
+
     }
 
     //_________________________________________________________
-    QtSettings::PathList QtSettings::kdeConfigPathList( void ) const
+    PathList QtSettings::kdeConfigPathList( void ) const
     {
 
         PathList out;
@@ -185,7 +190,12 @@ namespace Oxygen
         // load icon install prefix
         char* path = 0L;
         if( g_spawn_command_line_sync( "kde4-config --path config", &path, 0L, 0L, 0L ) && path )
-        { out.split( path ); };
+        {
+            out.split( path );
+        } else {
+            const std::string userConfigDir( std::string( g_get_user_config_dir() ) + "/oxygen-gtk" );
+            out.push_back( userConfigDir );
+        }
 
         out.push_back( GTK_THEME_DIR );
         return out;
@@ -193,7 +203,7 @@ namespace Oxygen
     }
 
     //_________________________________________________________
-    QtSettings::PathList QtSettings::kdeIconPathList( void ) const
+    PathList QtSettings::kdeIconPathList( void ) const
     {
 
         // load icon install prefix
@@ -202,7 +212,10 @@ namespace Oxygen
         if( g_spawn_command_line_sync( "kde4-config --path icon", &path, 0L, 0L, 0L ) && path )
         { out.split( path ); }
 
-        out.push_back( _defaultKdeIconPath );
+        // make sure defaultKdeIconPath is included in the list
+        if( std::find( out.begin(), out.end(), _defaultKdeIconPath ) == out.end() )
+        { out.push_back( _defaultKdeIconPath ); }
+
         return out;
 
     }
@@ -224,7 +237,7 @@ namespace Oxygen
         _iconThemes.insert( theme );
 
         #if OXYGEN_DEBUG
-        std::cout << "Oxygen::QtSettings::addIconTheme - adding " << theme << std::endl;
+        std::cerr << "Oxygen::QtSettings::addIconTheme - adding " << theme << std::endl;
         #endif
 
         // add all possible path (based on _kdeIconPathList) and look for possible parent
@@ -232,7 +245,13 @@ namespace Oxygen
         for( PathList::const_iterator iter = _kdeIconPathList.begin(); iter != _kdeIconPathList.end(); ++iter )
         {
 
-            pathList.push_back( sanitizePath( *iter + '/' + theme ) );
+            // create path and check for existence
+            std::string path( sanitizePath( *iter + '/' + theme ) );
+            struct stat st;
+            if( stat( path.c_str(), &st ) != 0 ) continue;
+
+            // add to path list
+            pathList.push_back( path );
             if( parentTheme.empty() )
             {
                 const std::string index( sanitizePath( *iter + '/' + theme + "/index.theme" ) );
@@ -244,7 +263,12 @@ namespace Oxygen
 
         // add parent if needed
         if( !parentTheme.empty() )
-        { addIconTheme( pathList, parentTheme ); }
+        {
+            // split using "," as a separator
+            PathList parentThemes( parentTheme, "," );
+            for( PathList::const_iterator iter = parentThemes.begin(); iter != parentThemes.end(); ++iter )
+            { addIconTheme( pathList, *iter ); }
+        }
 
         return;
 
@@ -268,15 +292,10 @@ namespace Oxygen
 
         // add option
         if( showIconsOnPushButton == "false" )
-        {
-            _rc.addToHeaderSection( "gtk-button-images = 0\n" );
-        }
-
-        // create icon translator, for stock icons
-        GtkIcons icons;
+        { _rc.addToHeaderSection( "gtk-button-images = 0\n" ); }
 
         // load icon sizes from kde
-        const int desktopIconSize( _kdeGlobals.getOption( "[DesktopIcons]", "Size" ).toInt( 48 ) );
+        // const int desktopIconSize( _kdeGlobals.getOption( "[DesktopIcons]", "Size" ).toInt( 48 ) );
         const int dialogIconSize( _kdeGlobals.getOption( "[DialogIcons]", "Size" ).toInt( 32 ) );
         const int panelIconSize( _kdeGlobals.getOption( "[PanelIcons]", "Size" ).toInt( 32 ) );
         const int mainToolbarIconSize( _kdeGlobals.getOption( "[MainToolbarIcons]", "Size" ).toInt( 22 ) );
@@ -284,31 +303,35 @@ namespace Oxygen
         const int toolbarIconSize( _kdeGlobals.getOption( "[ToolbarIcons]", "Size" ).toInt( 22 ) );
 
         // set gtk icon sizes
-        icons.setIconSize( "panel-menu", smallIconSize );
-        icons.setIconSize( "panel", panelIconSize );
-        icons.setIconSize( "gtk-small-toolbar", toolbarIconSize );
-        icons.setIconSize( "gtk-large-toolbar", mainToolbarIconSize );
-        icons.setIconSize( "gtk-dnd", desktopIconSize );
-        icons.setIconSize( "gtk-button", smallIconSize );
-        icons.setIconSize( "gtk-menu", smallIconSize );
-        icons.setIconSize( "gtk-dialog", dialogIconSize );
-        icons.setIconSize( "", smallIconSize );
+        _icons.setIconSize( "panel-menu", smallIconSize );
+        _icons.setIconSize( "panel", panelIconSize );
+        _icons.setIconSize( "gtk-small-toolbar", toolbarIconSize );
+        _icons.setIconSize( "gtk-large-toolbar", mainToolbarIconSize );
+        _icons.setIconSize( "gtk-dnd", mainToolbarIconSize );
+        _icons.setIconSize( "gtk-button", smallIconSize );
+        _icons.setIconSize( "gtk-menu", smallIconSize );
+        _icons.setIconSize( "gtk-dialog", dialogIconSize );
+        _icons.setIconSize( "", smallIconSize );
 
         // load translation table, generate full translation list, and path to gtk
-        icons.loadTranslations( sanitizePath( std::string( GTK_THEME_DIR ) + "/icons4" ) );
+        _icons.loadTranslations( sanitizePath( std::string( GTK_THEME_DIR ) + "/icons4" ) );
 
         // generate full path list
         PathList iconThemeList;
         addIconTheme( iconThemeList, _kdeIconTheme );
         addIconTheme( iconThemeList, _kdeFallbackIconTheme );
 
-        _rc.merge( icons.generate( iconThemeList ) );
+        _rc.merge( _icons.generate( iconThemeList ) );
 
     }
 
     //_________________________________________________________
     void QtSettings::loadKdePalette( void )
     {
+
+        if( _kdeColorsInitialized ) return;
+        _kdeColorsInitialized = true;
+
         _palette.clear();
         _palette.setColor( Palette::Active, Palette::Window, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Window]", "BackgroundNormal" ) ) );
         _palette.setColor( Palette::Active, Palette::WindowText, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Window]", "ForegroundNormal" ) ) );
@@ -316,37 +339,31 @@ namespace Oxygen
         _palette.setColor( Palette::Active, Palette::Button, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Button]", "BackgroundNormal" ) ) );
         _palette.setColor( Palette::Active, Palette::ButtonText, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Button]", "ForegroundNormal" ) ) );
 
-        _palette.setColor( Palette::Active, Palette::Focus, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Button]", "DecorationFocus" ) ) );
-        _palette.setColor( Palette::Active, Palette::Hover, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Button]", "DecorationHover" ) ) );
-
         _palette.setColor( Palette::Active, Palette::Selected, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Selection]", "BackgroundNormal" ) ) );
         _palette.setColor( Palette::Active, Palette::SelectedText, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Selection]", "ForegroundNormal" ) ) );
 
         _palette.setColor( Palette::Active, Palette::Tooltip, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Tooltip]", "BackgroundNormal" ) ) );
         _palette.setColor( Palette::Active, Palette::TooltipText, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Tooltip]", "ForegroundNormal" ) ) );
 
+        _palette.setColor( Palette::Active, Palette::Focus, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:View]", "DecorationFocus" ) ) );
+        _palette.setColor( Palette::Active, Palette::Hover, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:View]", "DecorationHover" ) ) );
+
         _palette.setColor( Palette::Active, Palette::Base, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:View]", "BackgroundNormal" ) ) );
         _palette.setColor( Palette::Active, Palette::BaseAlternate, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:View]", "BackgroundAlternate" ) ) );
         _palette.setColor( Palette::Active, Palette::Text, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:View]", "ForegroundNormal" ) ) );
-
         _palette.setColor( Palette::Active, Palette::NegativeText, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:View]", "ForegroundNegative" ) ) );
-        _palette.setColor( Palette::Active, Palette::ViewHover, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:View]", "DecorationHover" ) ) );
+
+        _palette.setColor( Palette::Active, Palette::ActiveWindowBackground, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[WM]", "activeBackground" ) ) );
+        _palette.setColor( Palette::Active, Palette::InactiveWindowBackground, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[WM]", "inactiveBackground" ) ) );
 
         // generate inactive and disabled palette from active, applying effects from kdeglobals
         _inactiveChangeSelectionColor = ( _kdeGlobals.getOption( "[ColorEffects:Inactive]", "ChangeSelectionColor" ).toVariant<std::string>( "false" ) == "true" );
         _palette.generate( Palette::Active, Palette::Inactive, ColorUtils::Effect( Palette::Inactive, _kdeGlobals ), _inactiveChangeSelectionColor );
         _palette.generate( Palette::Active, Palette::Disabled, ColorUtils::Effect( Palette::Disabled, _kdeGlobals ) );
 
-        // fill (overwrite) additional colors for disabled palette
-        _palette.setColor( Palette::Disabled, Palette::WindowText, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Window]", "ForegroundInactive" ) ) );
-        _palette.setColor( Palette::Disabled, Palette::ButtonText, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Button]", "ForegroundInactive" ) ) );
-        _palette.setColor( Palette::Disabled, Palette::SelectedText, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Selection]", "ForegroundInactive" ) ) );
-        _palette.setColor( Palette::Disabled, Palette::TooltipText, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:Tooltip]", "ForegroundInactive" ) ) );
-        _palette.setColor( Palette::Disabled, Palette::Text, ColorUtils::Rgba::fromKdeOption( _kdeGlobals.getValue( "[Colors:View]", "ForegroundInactive" ) ) );
-
         #if OXYGEN_DEBUG
-        std::cout << "Oxygen::QtSettings::loadKdePalette - palette: " << std::endl;
-        std::cout << _palette << std::endl;
+        std::cerr << "Oxygen::QtSettings::loadKdePalette - palette: " << std::endl;
+        std::cerr << _palette << std::endl;
         #endif
 
     }
@@ -356,7 +373,7 @@ namespace Oxygen
     {
 
         #if OXYGEN_DEBUG
-        std::cout << "Oxygen::QtSettings::generateGtkColors" << std::endl;
+        std::cerr << "Oxygen::QtSettings::generateGtkColors" << std::endl;
         #endif
 
         // customize gtk palette
@@ -364,17 +381,7 @@ namespace Oxygen
 
         // default colors
         _rc.setCurrentSection( Gtk::RC::defaultSection() );
-        if( applicationName().isMozilla() || applicationName().isGoogleChrome() || applicationName().isOpenOffice() )
-        {
-
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[NORMAL]", _palette.color( Palette::Window ) ) );
-
-        } else {
-
-            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[NORMAL]", backgroundBottomColor( _palette.color( Palette::Window ) ) ) );
-
-        }
-
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[NORMAL]", _palette.color( Palette::Window ) ) );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[PRELIGHT]", _palette.color( Palette::Window ) ) );
 
         if( applicationName().isMozilla() || applicationName().isGoogleChrome() )
@@ -422,13 +429,36 @@ namespace Oxygen
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[NORMAL]", _palette.color( Palette::ButtonText ) ) );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[ACTIVE]", _palette.color( Palette::ButtonText ) ) );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[PRELIGHT]", _palette.color( Palette::ButtonText ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Text ) ) );
-        _rc.addToRootSection( "class \"GtkButton\" style \"oxygen-buttons\"" );
-        _rc.addToRootSection( "class \"GtkToggleButton\" style \"oxygen-buttons\"" );
-        _rc.addToRootSection( "class \"GtkRadioButton\" style \"oxygen-buttons\"" );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::ButtonText ) ) );
+        _rc.addToRootSection( "class \"GtkOptionMenu\" style \"oxygen-buttons\"" );
+        _rc.addToRootSection( "widget_class \"*<GtkButton>.<GtkLabel>\" style \"oxygen-buttons\"" );
+
+        _rc.addSection( "oxygen-combobox", "oxygen-buttons" );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[NORMAL]", _palette.color( Palette::ButtonText ) ) );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[ACTIVE]", _palette.color( Palette::ButtonText ) ) );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[PRELIGHT]", _palette.color( Palette::ButtonText ) ) );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::ButtonText ) ) );        _rc.addToRootSection( "class \"*Button\" style \"oxygen-buttons\"" );
+        _rc.addToRootSection( "widget_class \"*<GtkComboBox>.<GtkCellView>\" style \"oxygen-combobox\"" );
+
+        // checkboxes and radio buttons
+        _rc.addSection( "oxygen-checkbox-buttons", "oxygen-buttons" );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[NORMAL]", _palette.color( Palette::WindowText ) ) );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[ACTIVE]", _palette.color( Palette::WindowText ) ) );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[PRELIGHT]", _palette.color( Palette::WindowText ) ) );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::WindowText ) ) );
+        _rc.addToRootSection( "widget_class \"*<GtkCheckButton>.<GtkLabel>\" style \"oxygen-checkbox-buttons\"" );
+
+        // progressbar labels
+        _rc.addSection( "oxygen-progressbar-labels", Gtk::RC::defaultSection() );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[NORMAL]", _palette.color( Palette::WindowText ) ) );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[ACTIVE]", _palette.color( Palette::WindowText ) ) );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[PRELIGHT]", _palette.color( Palette::WindowText ) ) );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::WindowText ) ) );
+
+        _rc.addToRootSection( "class \"GtkProgressBar\" style \"oxygen-progressbar-labels\"" );
 
         // menu items
-        _rc.addSection( "oxygen-menubar-item", Gtk::RC::defaultSection() );
+        _rc.addSection( "oxygen-menubar-item", "oxygen-menu-font" );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  text[NORMAL]", _palette.color( Palette::Text ) ) );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  fg[NORMAL]", _palette.color( Palette::WindowText ) ) );
 
@@ -464,17 +494,12 @@ namespace Oxygen
         }
 
         // text entries
-        /*
-        to match oxygen: the Window background is used for disabled entries,
-        whereas the 'base' background is used when enabled
-        */
         _rc.addSection( "oxygen-entry", Gtk::RC::defaultSection() );
         _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[NORMAL]", _palette.color( Palette::Base ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Window ) ) );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  base[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Window ) ) );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  bg[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Base ) ) );
+        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  base[INSENSITIVE]", _palette.color( Palette::Disabled, Palette::Base ) ) );
         _rc.addToRootSection( "class \"GtkSpinButton\" style \"oxygen-entry\"" );
         _rc.addToRootSection( "class \"GtkEntry\" style \"oxygen-entry\"" );
-        _rc.addToRootSection( "class \"GtkEntry\" style \"oxygen-entry\"" );
         _rc.addToRootSection( "widget_class \"*<GtkComboBoxEntry>.<GtkButton>\" style \"oxygen-entry\"" );
         _rc.addToRootSection( "widget_class \"*<GtkCombo>.<GtkButton>\" style \"oxygen-entry\"" );
 
@@ -503,7 +528,7 @@ namespace Oxygen
         } else {
 
             #if OXYGEN_DEBUG
-            std::cout << "Oxygen::QtSettings::loadKdeFonts - cannot load default font" << std::endl;
+            std::cerr << "Oxygen::QtSettings::loadKdeFonts - cannot load default font" << std::endl;
             #endif
 
         }
@@ -528,7 +553,7 @@ namespace Oxygen
             } else {
 
                 #if OXYGEN_DEBUG
-                std::cout << "Oxygen::QtSettings::loadKdeFonts - cannot load font named " << iter->second << std::endl;
+                std::cerr << "Oxygen::QtSettings::loadKdeFonts - cannot load font named " << iter->second << std::endl;
                 #endif
                 local = defaultFont;
 
@@ -540,21 +565,31 @@ namespace Oxygen
         }
 
         // pass fonts to RC
-        _rc.addToHeaderSection( Gtk::RCOption<std::string>( "gtk-font-name", fonts[FontInfo::Default] ) );
+        if( fonts[FontInfo::Default].isValid() )
+        { _rc.addToHeaderSection( Gtk::RCOption<std::string>( "gtk-font-name", fonts[FontInfo::Default] ) ); }
 
-        _rc.setCurrentSection( Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  font_name", fonts[FontInfo::Default] ) );
+        if( fonts[FontInfo::Default].isValid() )
+        {
+            _rc.setCurrentSection( Gtk::RC::defaultSection() );
+            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  font_name", fonts[FontInfo::Default] ) );
+        }
 
-        _rc.addSection( "oxygen-menu-font", Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  font_name", fonts[FontInfo::Menu] ) );
-        _rc.addToRootSection( "widget_class \"*<GtkMenuItem>.<GtkLabel>\" style \"oxygen-menu-font\"" );
+        if( fonts[FontInfo::Menu].isValid() )
+        {
+            _rc.addSection( "oxygen-menu-font", Gtk::RC::defaultSection() );
+            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  font_name", fonts[FontInfo::Menu] ) );
+            _rc.addToRootSection( "widget_class \"*<GtkMenuItem>.<GtkLabel>\" style \"oxygen-menu-font\"" );
+        }
 
-        _rc.addSection( "oxygen-toolbar-font", Gtk::RC::defaultSection() );
-        _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  font_name", fonts[FontInfo::ToolBar] ) );
-        _rc.addToRootSection( "widget_class \"*<GtkToolbar>.*\" style \"oxygen-toolbar-font\"" );
+        if( fonts[FontInfo::ToolBar].isValid() )
+        {
+            _rc.addSection( "oxygen-toolbar-font", Gtk::RC::defaultSection() );
+            _rc.addToCurrentSection( Gtk::RCOption<std::string>( "  font_name", fonts[FontInfo::ToolBar] ) );
+            _rc.addToRootSection( "widget_class \"*<GtkToolbar>.*\" style \"oxygen-toolbar-font\"" );
+        }
 
         #if OXYGEN_DEBUG
-        std::cout << "Oxygen::QtSettings::loadKdeFonts - done." << std::endl;
+        std::cerr << "Oxygen::QtSettings::loadKdeFonts - done." << std::endl;
         #endif
 
     }
@@ -564,7 +599,7 @@ namespace Oxygen
     {
 
         #if OXYGEN_DEBUG
-        std::cout << "Oxygen::QtSettings::loadKdeGlobalsOptions" << std::endl;
+        std::cerr << "Oxygen::QtSettings::loadKdeGlobalsOptions" << std::endl;
         #endif
 
         // toolbar style
@@ -585,7 +620,7 @@ namespace Oxygen
     {
 
         #if OXYGEN_DEBUG
-        std::cout << "Oxygen::QtSettings::loadOxygenOptions" << std::endl;
+        std::cerr << "Oxygen::QtSettings::loadOxygenOptions" << std::endl;
         #endif
 
         // contrast
