@@ -4,11 +4,6 @@
 * this file is part of the oxygen gtk engine
 * Copyright (c) 2010 Hugo Pereira Da Costa <hugo@oxygen-icons.org>
 *
-* inspired notably from kdelibs/kdeui/color/kcolorutils.h
-* Copyright (C) 2007 Matthew Woehlke <mw_triad@users.sourceforge.net>
-* Copyright (C) 2007 Thomas Zander <zander@kde.org>
-* Copyright (C) 2007 Zack Rusin <zack@kde.org>
-*
 * This  library is free  software; you can  redistribute it and/or
 * modify it  under  the terms  of the  GNU Lesser  General  Public
 * License  as published  by the Free  Software  Foundation; either
@@ -26,10 +21,12 @@
 */
 
 #include "oxygenapplicationname.h"
+#include "oxygengtkicons.h"
 #include "oxygengtkrc.h"
 #include "oxygenoption.h"
 #include "oxygenoptionmap.h"
 #include "oxygenpalette.h"
+#include "oxygenpathlist.h"
 
 #include <iostream>
 #include <sstream>
@@ -56,6 +53,9 @@ namespace Oxygen
         //! initialize
         void initialize( void );
 
+        //! initialize colors
+        void initializeColors( void );
+
         //! palette
         const Palette& palette( void ) const
         { return _palette; }
@@ -219,19 +219,6 @@ namespace Oxygen
         // get home directory
         std::string home( void ) const;
 
-        //! path list
-        class PathList: public std::vector<std::string>
-        {
-            public:
-
-            //! split string using provided separator and store
-            void split( const std::string&, const std::string& = ":" );
-
-            //! concatenate using provided separator
-            std::string join( const std::string& = ":" ) const;
-
-        };
-
         //! icon path
         PathList kdeConfigPathList( void ) const;
 
@@ -384,6 +371,14 @@ namespace Oxygen
 
         //@}
 
+        //! initialization flags
+        bool _initialized;
+        bool _kdeColorsInitialized;
+        bool _gtkColorsInitialized;
+
+        //! gtk icons generator
+        GtkIcons _icons;
+
         //! rc options (passed to gtk at the end of init
         Gtk::RC _rc;
 
