@@ -59,6 +59,10 @@ namespace Oxygen
         const QtSettings& settings( void ) const
         { return _settings; }
 
+        //! settings
+        QtSettings& settings( void )
+        { return _settings; }
+
         //! helper
         StyleHelper& helper( void )
         { return _helper; }
@@ -134,9 +138,9 @@ namespace Oxygen
         void renderToolBarHandle( GdkWindow*, GdkRectangle*, gint, gint, gint, gint, StyleOptions ) const;
 
         //! frame
-        void drawFloatFrame( cairo_t* context, GdkWindow*, GdkRectangle*, gint, gint, gint, gint, StyleOptions ) const;
-        void drawFloatFrame( GdkWindow* window, GdkRectangle* r, gint x, gint y, gint w, gint h, StyleOptions opt) const
-        { drawFloatFrame( 0L, window, r, x, y, w, h, opt ); }
+        void drawFloatFrame( cairo_t* context, GdkWindow*, GdkRectangle*, gint, gint, gint, gint, StyleOptions, Palette::Role = Palette::Window ) const;
+        void drawFloatFrame( GdkWindow* window, GdkRectangle* r, gint x, gint y, gint w, gint h, StyleOptions opt, Palette::Role role = Palette::Window) const
+        { drawFloatFrame( 0L, window, r, x, y, w, h, opt, role ); }
 
         //! button slab
         // void renderButtonSlab( GdkWindow*, GdkRectangle*, gint, gint, gint, gint, StyleOptions, TileSet::Tiles = TileSet::Ring );
@@ -184,7 +188,7 @@ namespace Oxygen
         void renderSelection( GdkWindow*, GdkRectangle*, gint, gint, gint, gint, TileSet::Tiles tiles, StyleOptions );
 
         //! arrow
-        void renderArrow( GdkWindow*, GdkRectangle*, GtkArrowType, gint, gint, gint, gint, QtSettings::ArrowSize = QtSettings::ArrowNormal, StyleOptions = Contrast ) const;
+        void renderArrow( GdkWindow*, GdkRectangle*, GtkArrowType, gint, gint, gint, gint, QtSettings::ArrowSize = QtSettings::ArrowNormal, StyleOptions = Contrast, Palette::Role = Palette::ButtonText ) const;
 
         //! slider groove
         void renderSliderGroove( GdkWindow*, GdkRectangle*, gint, gint, gint, gint, StyleOptions );
@@ -205,7 +209,7 @@ namespace Oxygen
         void renderTabBarFrame( GdkWindow*, GdkRectangle*, gint, gint, gint, gint, const Gtk::Gap&, StyleOptions );
 
         //! tree 'standard' expanders (that is: +!-)
-        void renderTreeExpander( GdkWindow*, GdkRectangle*, gint, gint, gint, gint, GtkExpanderStyle, StyleOptions ) const;
+        void renderTreeExpander( GdkWindow*, GdkRectangle*, gint, gint, gint, gint, GtkExpanderStyle, StyleOptions, Palette::Role ) const;
 
         //@}
 
@@ -215,8 +219,8 @@ namespace Oxygen
         //! render XShape window decoration mask
         void drawWindecoShapeMask( cairo_t* context, WinDeco::Options wopt, gint x, gint y, gint w, gint h );
 
-    	//! draw windeco button
-	    void drawWindecoButton(cairo_t*,WinDeco::ButtonType,WinDeco::ButtonStatus,WinDeco::Options, gint,gint,gint,gint);
+        //! draw windeco button
+        void drawWindecoButton(cairo_t*,WinDeco::ButtonType,WinDeco::ButtonStatus,WinDeco::Options, gint,gint,gint,gint);
 
         // adjust scrollbar hole, depending on orientation and buttons settings
         void adjustScrollBarHole( int& x, int& y, int& w, int& h, StyleOptions ) const;
@@ -300,7 +304,7 @@ namespace Oxygen
         //@{
 
         //! draw resize handles for window decoration
-        void renderWindowDots(Cairo::Context&, gint x, gint y, gint w, gint h, const ColorUtils::Rgba&, bool);
+        void renderWindowDots(Cairo::Context&, gint x, gint y, gint w, gint h, const ColorUtils::Rgba&, WinDeco::Options);
 
         //@}
 
