@@ -52,9 +52,11 @@ namespace Oxygen
         {
             shadow=(_type==ButtonClose)?
                 ColorUtils::Rgba( settings().palette().color( Palette::NegativeText ) ):
-                ColorUtils::Rgba( settings().palette().color( Palette::ViewHover ) );
+                ColorUtils::Rgba( settings().palette().color( Palette::Hover ) );
             shadow=darkColor(shadow);
             icon=shadow;
+        } else {
+            shadow=ColorUtils::Rgba(0,0,0);
         }
 
         GdkPixbuf *windecoButtonGlow( helper().windecoButtonGlow( shadow, int(scale) ) );
