@@ -9,24 +9,20 @@
 * Largely inspired from Qtcurve style
 * Copyright (C) Craig Drummond, 2003 - 2010 craig.p.drummond@gmail.com
 *
+* This  library is free  software; you can  redistribute it and/or
+* modify it  under  the terms  of the  GNU Lesser  General  Public
+* License  as published  by the Free  Software  Foundation; either
+* version 2 of the License, or( at your option ) any later version.
 *
-* Permission is hereby granted, free of charge, to any person obtaining a copy
-* of this software and associated documentation files (the "Software"), to
-* deal in the Software without restriction, including without limitation the
-* rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
-* sell copies of the Software, and to permit persons to whom the Software is
-* furnished to do so, subject to the following conditions:
+* This library is distributed  in the hope that it will be useful,
+* but  WITHOUT ANY WARRANTY; without even  the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+* Lesser General Public License for more details.
 *
-* The above copyright notice and this permission notice shall be included in
-* all copies or substantial portions of the Software.
-*
-* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
-* IN THE SOFTWARE.
+* You should have received a copy of the GNU Lesser General Public
+* License  along  with  this library;  if not,  write to  the Free
+* Software Foundation, Inc., 51  Franklin St, Fifth Floor, Boston,
+* MA 02110-1301, USA.
 */
 
 #include "oxygenwindowmanager.h"
@@ -46,6 +42,7 @@ namespace Oxygen
         _dragDistance( 4 ),
         _dragDelay( 500 ),
         _widget( 0L ),
+        _lastRejectedEvent( 0L ),
         _x(-1),
         _y(-1)
     { initializeBlackList(); }
@@ -61,10 +58,22 @@ namespace Oxygen
     void WindowManager::registerWidget( GtkWidget* widget )
     {
 
-        if( _map.contains( widget ) ) return;
+        if( _map.contains( widget ) || widgetIsBlackListed( widget ) ) return;
+
+        // Window with no decorations (set by app), let window manage it self
+        if( GTK_IS_WINDOW( widget ) && !gtk_window_get_decorated( GTK_WINDOW( widget ) ) ) return;
+
+        // widgets used in tabs also must be ignored (happens, unfortunately)
+        GtkWidget* parent( gtk_widget_get_parent( widget ) );
+        if( GTK_IS_NOTEBOOK( parent ) && Gtk::gtk_notebook_is_tab_label( GTK_NOTEBOOK( parent ), widget ) )
+        { return; }
 
         #if OXYGEN_DEBUG
-        std::cout << "Oxygen::WindowManager::registerWidget - " << widget << "(" << G_OBJECT_TYPE_NAME( widget ) << ")" << std::endl;
+        std::cerr
+            << "Oxygen::WindowManager::registerWidget -"
+            << " " << widget << " (" << G_OBJECT_TYPE_NAME( widget ) << ")"
+            << " " << gtk_widget_get_name( widget )
+            << std::endl;
         #endif
 
         // Force widget to listen to relevant events
@@ -88,7 +97,7 @@ namespace Oxygen
         if( !_map.contains( widget ) ) return;
 
         #if OXYGEN_DEBUG
-        std::cout << "Oxygen::WindowManager::unregisterWidget - " << widget << "(" << G_OBJECT_TYPE_NAME( widget ) << ")" << std::endl;
+        std::cerr << "Oxygen::WindowManager::unregisterWidget - " << widget << " (" << G_OBJECT_TYPE_NAME( widget ) << ")" << std::endl;
         #endif
 
         _map.value( widget ).disconnect( widget );
@@ -114,7 +123,7 @@ namespace Oxygen
         else if( _mode == Disabled )
         {
             DataMap<Data>::Map& map( _map.map() );
-            for( DataMap<Data>::Map::iterator iter = map.begin(); iter != map.end(); iter++ )
+            for( DataMap<Data>::Map::iterator iter = map.begin(); iter != map.end(); ++iter )
             { connect( iter->first, iter->second ); }
         }
 
@@ -144,23 +153,38 @@ namespace Oxygen
         if( event->type == GDK_BUTTON_PRESS && event->button == Gtk::LeftButton )
         {
 
+            const bool accepted( static_cast<WindowManager*>(data)->isWindowDragWidget( widget, event ) );
             #if OXYGEN_DEBUG
-            std::cout << "Oxygen::WindowManager::wmButtonPress - "
-                << widget
-                << "(" << G_OBJECT_TYPE_NAME( widget ) << ")"
+            std::cerr << "Oxygen::WindowManager::wmButtonPress -"
+                << " event: " << event
+                << " widget: " << widget
+                << " (" << G_OBJECT_TYPE_NAME( widget ) << ")"
                 << " " << gtk_widget_get_name( widget )
+                << " accepted: " << accepted
                 << std::endl;
             #endif
 
-            return static_cast<WindowManager*>(data)->isWindowDragWidget( widget, event );
+            return accepted;
 
         } else return false;
 
     }
 
     //_________________________________________________
-    gboolean WindowManager::wmButtonRelease(GtkWidget *widget, GdkEventButton*, gpointer data )
-    { return static_cast<WindowManager*>( data )->finishDrag( widget ); }
+    gboolean WindowManager::wmButtonRelease(GtkWidget *widget, GdkEventButton* event, gpointer data )
+    {
+
+        #if OXYGEN_DEBUG
+        std::cerr << "Oxygen::WindowManager::wmButtonRelease -"
+            << " event: " << event
+            << " widget: " << widget
+            << " (" << G_OBJECT_TYPE_NAME( widget ) << ")"
+            << " " << gtk_widget_get_name( widget )
+            << std::endl;
+        #endif
+
+        return static_cast<WindowManager*>( data )->finishDrag( widget );
+    }
 
     //_________________________________________________
     gboolean WindowManager::wmLeave(GtkWidget *widget, GdkEventCrossing*, gpointer data )
@@ -197,7 +221,13 @@ namespace Oxygen
             _drag = true;
             return true;
 
-        } else return false;
+        } else {
+
+            // mark event as rejected
+            _lastRejectedEvent = event;
+            return false;
+        }
+
     }
 
     //_________________________________________________________________
@@ -257,6 +287,7 @@ namespace Oxygen
     {
 
         _widget = 0L;
+        _lastRejectedEvent = 0L;
         _x = -1;
         _y = -1;
 
@@ -276,50 +307,79 @@ namespace Oxygen
     bool WindowManager::withinWidget( GtkWidget* widget, GdkEventButton* event ) const
     {
 
-        // get widget window
-        GdkWindow *window( gtk_widget_get_parent_window( widget ) );
+        // get top level widget
+        GtkWidget* topLevel( gtk_widget_get_toplevel( widget ) );
+        if( !topLevel ) return true;
 
-        // Some widgets aren't realized (GeditWindow for exemple) ...
+        // get top level window;
+        GdkWindow *window( gtk_widget_get_window( topLevel ) );
         if( !window ) return true;
 
-        GtkAllocation allocation;
-        #if GTK_CHECK_VERSION(2, 18, 0)
-        gtk_widget_get_allocation( widget, &allocation );
-        #else
-        allocation = widget->allocation;
-        #endif
+        // translate widget position to topLevel
+        int wx(0);
+        int wy(0);
+        gtk_widget_translate_coordinates( widget, topLevel, wx, wy, &wx, &wy );
 
-        // Need to get absolute coordinates
+        // translate to absolute coordinates
         int nx(0);
         int ny(0);
-        gdk_window_get_origin(window, &nx, &ny );
-        allocation.x += nx;
-        allocation.y += ny;
+        gdk_window_get_origin( window, &nx, &ny );
+        wx += nx;
+        wy += ny;
 
+        // get widget size.
+        // for notebooks, only consider the tabbar rect
+        GtkAllocation allocation;
+        if( GTK_IS_NOTEBOOK( widget ) )
+        {
+
+            Gtk::gtk_notebook_get_tabbar_rect( GTK_NOTEBOOK( widget ), &allocation );
+            allocation.x += wx - widget->allocation.x;
+            allocation.y += wy - widget->allocation.y;
+
+        } else {
+
+            allocation = widget->allocation;
+            allocation.x = wx;
+            allocation.y = wy;
+
+        }
+
+        // compare to event root position
         return Gtk::gdk_rectangle_contains( &allocation, int(event->x_root), int(event->y_root) );
 
     }
 
     //_________________________________________________
-    bool WindowManager::useEvent( GtkWidget* widget, GdkEventButton* event ) const
+    bool WindowManager::useEvent( GtkWidget* widget, GdkEventButton* event )
     {
 
         // check against mode
         if( _mode == Disabled ) return false;
         if( _mode == Minimal && !( GTK_IS_TOOLBAR( widget ) || GTK_IS_MENU_BAR( widget ) ) ) return false;
+        if( _lastRejectedEvent && event == _lastRejectedEvent ) return false;
 
         // always accept if widget is not a container
         if( !GTK_IS_CONTAINER( widget ) ) return true;
 
         // if widget is a notebook, accept if there is no hovered tab
+        bool useEvent( true );
         if( GTK_IS_NOTEBOOK( widget ) )
         {
-            return
+
+            useEvent =
+                ( !Gtk::gtk_notebook_has_visible_arrows( GTK_NOTEBOOK( widget ) ) ) &&
                 Style::instance().animations().tabWidgetEngine().contains( widget ) &&
-                Style::instance().animations().tabWidgetEngine().hoveredTab( widget ) == -1;
+                Style::instance().animations().tabWidgetEngine().hoveredTab( widget ) == -1 &&
+                childrenUseEvent( widget, event, false );
+
+        } else {
+
+            useEvent = childrenUseEvent( widget, event, false );
+
         }
 
-        return childrenUseEvent( widget, event, false );
+        return useEvent;
 
     }
 
@@ -339,62 +399,49 @@ namespace Oxygen
             GtkWidget* childWidget( GTK_WIDGET( child->data ) );
 
             // check widget state and type
-            if( gtk_widget_get_state( childWidget ) == GTK_STATE_PRELIGHT ) {
+            if( gtk_widget_get_state( childWidget ) == GTK_STATE_PRELIGHT )
+            {
 
                 // if widget is prelight, we don't need to check where event happen,
                 // any prelight widget indicate we can't do a move
                 usable = false;
+                continue;
 
-            } else if( GTK_IS_NOTEBOOK( childWidget ) ) {
-
-                inNoteBook = true;
-
-            } else if( event && withinWidget( childWidget, event ) ) {
-
-                #if OXYGEN_DEBUG
-                std::cout << "Oxygen::WindowManager::childrenUseEvent - "
-                    << widget
-                    << "(" << G_OBJECT_TYPE_NAME( childWidget ) << ")"
-                    << " " << gtk_widget_get_name( childWidget )
-                    << std::endl;
-                #endif
-
-                GdkWindow *window = gtk_widget_get_window( childWidget );
-                if( window && gdk_window_is_visible ( window ) )
-                {
-
-                    // check against black-list
-                    if( widgetIsBlackListed( childWidget ) )
-                    {
-
-                        usable = false;
-
-                    } else if( gtk_widget_get_events ( childWidget ) & (GDK_BUTTON_PRESS_MASK|GDK_BUTTON_RELEASE_MASK) ) {
-
-                        // widget listening to press event
-                        usable = false;
-
-                    } else if( GTK_IS_MENU_ITEM( childWidget ) ) {
-
-                        // deal with menu item, GtkMenuItem only listen to
-                        // GDK_BUTTON_PRESS_MASK when state == GTK_STATE_PRELIGHT
-                        // so previous check are invalids :(
-                        usable = false;
-
-                    } else if( GTK_IS_SCROLLED_WINDOW( childWidget ) && ( !inNoteBook || gtk_widget_is_focus( childWidget ) ) ) {
-
-                        // Scrolled do not send release events
-                        // to parents so not usable
-                        usable = false;
-
-                    }
-                }
             }
 
+            GdkWindow *window = gtk_widget_get_window( childWidget );
+            if( !( window && gdk_window_is_visible( window ) ) )
+            { continue; }
+
+            if( GTK_IS_NOTEBOOK( childWidget ) ) inNoteBook = true;
+
+            if( !( event && withinWidget( childWidget, event ) ) )
+            { continue; }
+
+            // check special cases for which grab should not be enabled
+            if(
+                ( widgetIsBlackListed( childWidget ) ) ||
+                ( GTK_IS_NOTEBOOK( widget ) && Gtk::gtk_notebook_is_tab_label( GTK_NOTEBOOK( widget ), childWidget ) ) ||
+                ( GTK_IS_BUTTON( childWidget ) && gtk_widget_get_state( childWidget ) != GTK_STATE_INSENSITIVE ) ||
+                ( gtk_widget_get_events ( childWidget ) & (GDK_BUTTON_PRESS_MASK|GDK_BUTTON_RELEASE_MASK) ) ||
+                ( GTK_IS_MENU_ITEM( childWidget ) ) ||
+                ( GTK_IS_SCROLLED_WINDOW( childWidget ) && ( !inNoteBook || gtk_widget_is_focus( childWidget ) ) ) )
+            { usable = false; }
+
             // if child is a container and event has been accepted so far, also check it, recursively
             if( usable && GTK_IS_CONTAINER( childWidget ) )
             { usable = childrenUseEvent( childWidget, event, inNoteBook); }
 
+            #if OXYGEN_DEBUG
+            std::cerr << "Oxygen::WindowManager::childrenUseEvent -"
+                << " event: " << event
+                << " widget: " << childWidget
+                << " (" << G_OBJECT_TYPE_NAME( childWidget ) << ")"
+                << " " << gtk_widget_get_name( childWidget )
+                << " usable: " << usable
+                << std::endl;
+            #endif
+
         }
 
         // free list
@@ -409,7 +456,10 @@ namespace Oxygen
         // clear list
         _blackList.clear();
         _blackList.push_back( "GtkPizza" );
+        _blackList.push_back( "MetaFrames" );
         _blackList.push_back( "GladeDesignLayout" );
+        _blackList.push_back( "SPHRuler" );
+        _blackList.push_back( "SPVRuler" );
     }
 
     //________________________________________________________________________________
